const axios = require("axios");
const moment = require("moment");
const _ = require("lodash");
const getRequest = require("../helpers/request");
const getRatingByField = require("../helpers/getRatingByField");

const COLLECTION_TASK_CHECKS = "task_checks";
const COLLECTION_SECTION_QUALITY = "section_quality_time_limits";
const COLLECTION_METRIC_QUALITY = "check_metric_quality_limits";
const COLLECTION_CHECK_PRIORITIES = "check_priorities";

const RatingService = async (req, res) => {
  try {
    const { taskId, write } = req.params;
    const task = await getRequest(`tasks/${taskId}`); // это задача КК!

    if (!task.isQuality) {
      throw new Error(`Not a quality task [${taskId}]`);
    }

    if (!task.qualityTaskId || _.isEmpty(task.qualityTask)) {
      throw new Error(`Missing checked task [${taskId}]`);
    }

    const checkedTask = task.qualityTask; // это проверяемая задача!

    const {
      sectionId, // проверяемый вид работ
      workerId, // Проверяемый сотрудник
    } = checkedTask;

    const paramsTaskChecks = {
      itemsPerPage: 100,
      controllerTaskId: taskId,
    };

    // Собранные метрики из task, т.е. в задаче КК
    const taskChecks = await getRequest(
      COLLECTION_TASK_CHECKS,
      paramsTaskChecks,
    ).then((data) => data["hydra:member"]);

    // Лимиты по времени выполнения проверяемого вида работ
    const sectionTimeLimit = await getRequest(COLLECTION_SECTION_QUALITY, {
      sectionId,
      category: task.rowType,
    }).then((data) => data["hydra:member"]);

    // Собираем лимиты по метрикам
    for (const taskCheck of taskChecks) {
      // По каждой метрике делаем запрос лимитов, расчитываем рейтинг, пишем в род объект
      const metricLimits = await getRequest(COLLECTION_METRIC_QUALITY, {
        checkMetricId: taskCheck.checkMetricId,
      }).then((data) => data["hydra:member"]);

      taskCheck.ratingScore = getRatingByField({
        limits: metricLimits,
        value: taskCheck.value,
        minNameField: "minQuality",
        maxNameField: "maxQuality",
      });

      if (write === "1") {
        await axios({
          method: "patch",
          url: `${process.env.API_HOST}/${COLLECTION_TASK_CHECKS}/${taskCheck.id}`,
          headers: {
            "X-Username": "agrocacher [PATCH TASK_CHECKS]",
            "Content-Type": "application/merge-patch+json",
          },
          data: {
            ratingScore: Number(taskCheck.ratingScore),
          },
        });
      }
    }

    const ratingScore = _.min(taskChecks.map(({ ratingScore }) => ratingScore)) || 0;

    const finishedAt = moment(checkedTask.finishedAt);
    const startedAt = moment(checkedTask.startedAt);
    const diff = finishedAt.diff(startedAt, "seconds");

    const ratingTime = getRatingByField({
      limits: sectionTimeLimit,
      value: diff,
      minNameField: "minTime",
      maxNameField: "maxTime",
    });
    const dynamicRating = (ratingScore * 0.6 + ratingTime * 0.4).toFixed(1) || 0;
    if (write === "1") {
      // Прописываем его в Task у проверяемой (!!) задачи. В задачу КК писать не будем,
      // чтобы нам потом не высчитывался общий рейтинг по контролю качества у контроллера
      await axios({
        method: "patch",
        url: `${process.env.API_HOST}/tasks/${checkedTask.id}`,
        headers: {
          "X-Username": "agrocacher [PATCH TASKS]",
          "Content-Type": "application/merge-patch+json",
        },
        data: {
          ratingTime: Number(ratingTime),
          ratingScore: Number(ratingScore),
          dynamicRating: Number(dynamicRating),
        },
      });

      const checkPriority = await getRequest(COLLECTION_CHECK_PRIORITIES, {
        sectionId,
        workerId,
      }).then((data) => data["hydra:member"][0]);
      // Прописываем lastCheck в таблице приоритетов у сотрудника дату завершения задачи КК, если она старее текущего значения lastCheck
      if (finishedAt.isAfter(checkPriority.lastCheck)) {
        await axios({
          method: "patch",
          url: `${process.env.API_HOST}/check_priorities/${checkPriority.id}`,
          headers: {
            "X-Username": "agrocacher [PATCH CHECK_PRIORITY]",
            "Content-Type": "application/merge-patch+json",
          },
          data: {
            lastCheck: finishedAt.format(),
          },
        });
      }

      // Возможно, стоит еще приоритет поменять
    }

    return res.json({
      error: false,
      saved: write === "1",
      controller: task.workerName, // проверяющий
      worker: checkedTask.workerName, // проверяемый
      workerId,
      section: checkedTask.sectionName, // проверяемый вид работ
      ratingTime, // рейтинг
      ratingScore, // рейтинг
      dynamicRating,
      task: taskId, // ID задачи КК
      checkedTask: checkedTask.id, // ID проверяемой задачи
      metrics: _.map(taskChecks, (t) => ({
        name: t.checkMetric.name,
        value: t.value,
        ratingScore: t.ratingScore,
      })), // снятые метрики
    });
  } catch (e) {
    console.error(e);

    return res.status(500).send({
      error: true,
      message: `RATING ERROR! [${e}]`,
    });
  }
};

module.exports = RatingService;
